@@ -25,6 +25,17 @@
  * Boston, MA  02111-1307, USA.
  */
 
+/*
+ * Modified 28/11/2001 Iustin Pop <iusty@intensit.de>
+ * There was a bug regarding the LDAP_TLS option: if both LDAP_TLS
+ * and was LDAP_AUTHBIND were enabled, the ldap_start_tls function
+ * was called only for the first connection, resulting in the fact
+ * that the bind for checking the password was done without TLS,
+ * sending the password in clear text over the network. Detected 
+ * when using OpenLDAP with "security ssf=128" (which disalows any 
+ * clear-text communication).
+*/
+
 /*
    Modified 01/21/2000 James Golovich <james@wwnet.net>
 
@@ -32,8 +43,6 @@
 handle passwords via authenticated binds, instead of checking these
 internally.
 2. Changed paramaters for authldap_get to include pass.
-3. Added int authbind and int authbind_ok to struct ldap_info to
-handle authenticated binds.
 
 */
 
@@ -118,6 +127,7 @@ struct ldap_info
 	const char *bindpw;
 	const char *basedn;
 	const char *mail;
+        const char *filter;
 	const char *domain;
 	uid_t uid;
 	gid_t gid;
@@ -125,7 +135,13 @@ struct ldap_info
 	int   authbind;
 	int   deref;
 	int   tls;
-	int	authbind_ok;  /* not used? */
+
+	/* Optional emailmap to handle */
+
+	const char *emailmap;
+	const char *emailmap_basedn;
+	const char *emailmap_handle;
+	const char *emailmap_handle_lookup;
 };
 
 /*
@@ -145,7 +161,27 @@ static void ldapconnfailure()
 {
 	const char *p=getenv("LDAP_MEMORY_LEAK");
 
-	if (!p) p="1";
+	if (!p)
+	{
+#ifdef LDAP_VENDOR_NAME
+#ifdef LDAP_VENDOR_VERSION
+#define DO_OPENLDAP_CHECK
+#endif
+#endif
+
+#ifdef DO_OPENLDAP_CHECK
+
+		/* It's supposed to be fixed in 20019 */
+
+		if (strcmp(LDAP_VENDOR_NAME, "OpenLDAP") == 0 &&
+		    LDAP_VENDOR_VERSION < 20019)
+			p="1";
+		else
+			p="0";
+#else
+		p="0";
+#endif
+	}
 
 	if (atoi(p) && !ldapfailflag)
 	{
@@ -364,6 +400,13 @@ static int authldap_read_config(struct ldap_info *ldap)
     ldap->tls=atoi(p);
   }
 
+  ldap->filter=0;
+  p=0;
+  if (read_env("LDAP_FILTER", &p, "", 0, "") && p)
+  {
+    ldap->filter=p;
+  }
+
   ldap->deref = LDAP_DEREF_NEVER; 
   p=0;
   if (!read_env("LDAP_DEREF", &p, "", 0, ""))
@@ -380,6 +423,13 @@ static int authldap_read_config(struct ldap_info *ldap)
 	  ldap->deref = LDAP_DEREF_ALWAYS; 
   }
 
+  if (!read_env("LDAP_EMAILMAP", &ldap->emailmap, "", 0, "") ||
+      !read_env("LDAP_EMAILMAP_BASEDN", &ldap->emailmap_basedn, "", 0, "") ||
+      !read_env("LDAP_EMAILMAP_ATTRIBUTE", &ldap->emailmap_handle, "", 0, "")||
+      !read_env("LDAP_EMAILMAP_MAIL",
+		&ldap->emailmap_handle_lookup, "", 0, ""))
+	return (0);
+
   return 1;
 }
 
@@ -402,7 +452,7 @@ static void copy_value(LDAP *ld, LDAPMessage *entry, const char *attribut,
 
 	if (values==NULL)
 	{
-#ifdef OPENLDAPV2
+#ifdef HAVE_LDAP_RESULT2ERROR
 	  int ld_errno = ldap_result2error(ld,entry,0);
 	  if (ld_errno && ld_errno != LDAP_DECODING_ERROR)
 	    /* We didn't ask for this attribute */
@@ -467,6 +517,53 @@ static int ldaperror(int rc)
 	return (rc);
 }
 
+/* This function takes a ldap connection and 
+ * tries to enable TLS on it.
+*/
+static int enable_tls_on(LDAP *conn) {
+#if HAVE_LDAP_TLS
+	int version;
+	int ldrc;
+
+	if (ldaperror(ldrc=ldap_get_option (conn,
+				    LDAP_OPT_PROTOCOL_VERSION,
+				    &version))
+	    != LDAP_SUCCESS)
+	{
+		const char *s=ldap_err2string(ldrc);
+
+#if	HAVE_SYSLOG_H
+		syslog(LOG_DAEMON|LOG_CRIT,
+		       "ldap_get_option failed: %s", s);
+#endif
+		return (-1);
+	}
+
+	if (version < LDAP_VERSION3)
+	{
+		version = LDAP_VERSION3;
+		(void)ldap_set_option (conn,
+				       LDAP_OPT_PROTOCOL_VERSION,
+				       &version);
+	}
+
+	if (ldaperror(ldrc=ldap_start_tls_s(conn, NULL, NULL))
+	    != LDAP_SUCCESS)
+	{
+		const char *s=ldap_err2string(ldrc);
+
+#if	HAVE_SYSLOG_H
+		syslog(LOG_DAEMON|LOG_CRIT,
+		       "ldap_start_tls_s failed: %s", s);
+#endif
+		return (-1);
+	}
+	return 0;
+#else
+	return (-1);
+#endif
+}
+
 static LDAP *ldapconnect()
 {
 LDAP	*p;
@@ -518,47 +615,11 @@ int     ldrc;
 	}
 
 #if HAVE_LDAP_TLS
-	if (my_ldap.tls)
+	if (my_ldap.tls && enable_tls_on(my_ldap_fp))
 	{
-		int version;
-
-		if (ldaperror(ldrc=ldap_get_option (my_ldap_fp,
-						    LDAP_OPT_PROTOCOL_VERSION,
-						    &version))
-		    != LDAP_SUCCESS)
-		{
-			const char *s=ldap_err2string(ldrc);
-
-#if	HAVE_SYSLOG_H
-			syslog(LOG_DAEMON|LOG_CRIT,
-			       "ldap_get_option failed: %s", s);
-#endif
-			authldapclose();
-			ldapconnfailure();
-			return (-1);
-		}
-
-		if (version < LDAP_VERSION3)
-		{
-			version = LDAP_VERSION3;
-			(void)ldap_set_option (my_ldap_fp,
-					       LDAP_OPT_PROTOCOL_VERSION,
-					       &version);
-		}
-
-		if (ldaperror(ldrc=ldap_start_tls_s(my_ldap_fp, NULL, NULL))
-		    != LDAP_SUCCESS)
-		{
-			const char *s=ldap_err2string(ldrc);
-
-#if	HAVE_SYSLOG_H
-			syslog(LOG_DAEMON|LOG_CRIT,
-			       "ldap_start_tls_s failed: %s", s);
-#endif
-			authldapclose();
-			ldapconnfailure();
-			return (-1);
-		}
+		authldapclose();
+		ldapconnfailure();
+		return (-1);
 	}
 #endif
 
@@ -632,9 +693,137 @@ int authldapcommon(const char *user, const char *pass,
 	return (auth_ldap_do(user, pass, callback, arg, NULL));
 }
 
+static int auth_ldap_do2(const char *user, const char *pass,
+			int (*callback)(struct authinfo *, void *),
+			 void *arg, const char *newpass);
+
+static char *escape_str(const char *);
+
 static int auth_ldap_do(const char *user, const char *pass,
 			int (*callback)(struct authinfo *, void *),
                         void *arg, const char *newpass)
+{
+	char *q;
+	int i;
+
+	q=escape_str(user);
+
+	if (!q)
+		return (auth_ldap_do2(user, pass, callback, arg, newpass));
+
+	i=auth_ldap_do2(q, pass, callback, arg, newpass);
+	free(q);
+	return (i);
+}
+
+
+static int auth_ldap_do3(const char *attrname,
+			 const char *user, const char *pass,
+			 int (*callback)(struct authinfo *, void *),
+			 void *arg, const char *newpass, const char *authaddr);
+
+static char *emailmap_get_search_string(const char *str, const char *email);
+
+static int auth_ldap_do2(const char *user, const char *pass,
+			int (*callback)(struct authinfo *, void *),
+                        void *arg, const char *newpass)
+{
+	char *srch;
+	struct timeval tv;
+	const char *attributes[2];
+	LDAPMessage *result, *entry;
+	int cnt;
+	char *v;
+	const char *aname;
+
+	if (ldapopen())	return (1);
+
+	if (my_ldap.emailmap[0] == 0 || strchr(user, '@') == NULL)
+		return (auth_ldap_do3(my_ldap.mail,
+				      user, pass, callback, arg, newpass,
+				      user));
+	/* Mapping is not enabled */
+
+	srch=emailmap_get_search_string(my_ldap.emailmap, user);
+
+	if (!srch)
+	{
+		perror("malloc");
+		exit(1);
+	}
+
+	tv.tv_sec=my_ldap.timeout;
+	tv.tv_usec=0;
+
+	attributes[0]=my_ldap.emailmap_handle;
+
+	if (!attributes[0][0])
+		attributes[0]="handle";
+	attributes[1]=NULL;
+
+	if (ldaperror(ldap_search_st(my_ldap_fp,
+				     (char *)(my_ldap.emailmap_basedn[0] ?
+					      my_ldap.emailmap_basedn
+					      :my_ldap.basedn),
+				     LDAP_SCOPE_SUBTREE,
+				     srch, (char **)attributes, 0,
+				     &tv, &result)
+		      != LDAP_SUCCESS))
+	{
+		free(srch);
+
+		if (my_ldap_fp)	return (-1);
+		return (1);
+	}
+
+	if ((cnt=ldap_count_entries(my_ldap_fp, result)) != 1)
+	{
+		free(srch);
+
+		if (cnt)
+			syslog(LOG_DAEMON|LOG_CRIT,
+			       "authldap: %d results from search: %s\n",
+			       cnt, srch);
+		ldap_msgfree(result);
+		return -1;
+	}
+	free(srch);
+
+	entry=ldap_first_entry(my_ldap_fp, result);
+
+	if (!entry)
+	{
+		ldap_msgfree(result);
+
+		syslog(LOG_DAEMON|LOG_CRIT,
+		       "authldap: unexpected NULL from ldap_first_entry");
+		return -1;
+	}
+
+	copy_value(my_ldap_fp, entry, attributes[0], &v, user);
+
+	if (!v)
+	{
+		ldap_msgfree(result);
+		return (-1);
+	}
+
+	aname=my_ldap.emailmap_handle_lookup;
+	if (aname[0] == 0)
+		aname=my_ldap.mail;
+
+	cnt=auth_ldap_do3(aname, v, pass, callback, arg, newpass, user);
+
+	ldap_msgfree(result);
+	free(v);
+	return (cnt);
+}
+
+static int auth_ldap_do3(const char *attrname,
+			 const char *user, const char *pass,
+			 int (*callback)(struct authinfo *, void *),
+			 void *arg, const char *newpass,
+			 const char *authaddr)
 {
 	char *newpass_crypt=0;
 	const char *attributes[10], *ldap_attributes[10];
@@ -656,23 +845,46 @@ static int auth_ldap_do(const char *user, const char *pass,
 	gid_t ag;
 	int rc;
 	char *quota=0;
+        int additionalFilter = 0;
+        int hasAdditionalFilter = 0;
+
+        hasAdditionalFilter = my_ldap.filter != 0;
 
 	memset(&auth, 0, sizeof(auth));
 
-	if (ldapopen())	return (1);
+        if (hasAdditionalFilter)
+        {
+            /* To add the additional filter, we need to add on the
+             * additional size for "(&)" and the other filter.  So
+             * filter+3
+             */
+            additionalFilter = strlen(my_ldap.filter) + 3;
+        }
 
-	if ((filter=malloc(strlen(my_ldap.mail)+strlen(user)+
+	if ((filter=malloc(additionalFilter+strlen(attrname)+strlen(user)+
 			   (my_ldap.domain ? strlen(my_ldap.domain):0)+
 			   sizeof ("(=@)"))) == 0)
 	{
 		perror("malloc");
 		return 1;
 	}
+        strcpy(filter, "\0");
 
-	strcat(strcat(strcat(strcpy(filter, "("), my_ldap.mail), "="), user);
+        if (hasAdditionalFilter)
+        {
+            strcat(filter, "(&");
+            strcat(filter, my_ldap.filter);
+        }
+
+        strcat(strcat(strcat(strcat(filter, "("), attrname), "="), user);
 	if ( my_ldap.domain && my_ldap.domain[0] && strchr(user, '@') == 0 )
 		strcat(strcat(filter, "@"), my_ldap.domain);
 	strcat(filter, ")");
+        
+        if (hasAdditionalFilter)
+        {
+            strcat(filter, ")");
+        }
 
 	timeout.tv_sec=my_ldap.timeout;
 	timeout.tv_usec=0;
@@ -704,8 +916,8 @@ static int auth_ldap_do(const char *user, const char *pass,
 	{
 		free(filter);
 
-		if (my_ldap_fp)	return (1);
-		return (-1);
+		if (my_ldap_fp)	return (-1);
+		return (1);
 	}
 
 	free(filter);
@@ -762,11 +974,11 @@ static int auth_ldap_do(const char *user, const char *pass,
 		unsigned long n;
 
 		copy_value(my_ldap_fp, entry, attributes[5], &p, user);
-		if (!p)	return (0);
-
-		if (sscanf(p, "%lu", &n) > 0)
-			au= (uid_t)n;
-		free(p);
+		if (p) {
+			if (sscanf(p, "%lu", &n) > 0)
+				au= (uid_t)n;
+			free(p);
+		}
 #if DEBUG_LDAP
 		syslog(LOG_DAEMON|LOG_CRIT,"au= %d\n",au);
 #endif
@@ -778,11 +990,11 @@ static int auth_ldap_do(const char *user, const char *pass,
 		unsigned long n;
 
 		copy_value(my_ldap_fp, entry, attributes[6], &p, user);
-		if (!p)	return (0);
-
-		if (sscanf(p, "%lu", &n) > 0)
-			ag= (gid_t)n;
-		free(p);
+		if (p) {
+			if (sscanf(p, "%lu", &n) > 0)
+				ag= (gid_t)n;
+			free(p);
+		}
 #if DEBUG_LDAP
 		syslog(LOG_DAEMON|LOG_CRIT,"ag= %d\n",ag);
 #endif
@@ -795,7 +1007,7 @@ static int auth_ldap_do(const char *user, const char *pass,
 	auth.sysuserid= &au;
 	auth.sysgroupid= ag;
 	auth.homedir=homeDir;
-	auth.address=user;
+	auth.address=authaddr;
 	auth.fullname=cn;
 	auth.maildir=mailDir;
 	auth.clearpasswd=userPassword;
@@ -828,17 +1040,28 @@ static int auth_ldap_do(const char *user, const char *pass,
 				rc=1;
 			else
 			{
-				switch (ldap_simple_bind_s(bindp, dn, (char *)pass))
-				{
-				case LDAP_SUCCESS:
-					break;
-				case LDAP_INVALID_CREDENTIALS:
-					rc = -1;
-					break;
-				default:
+#if HAVE_LDAP_TLS
+				if(my_ldap.tls && enable_tls_on(bindp)) {
+#if HAVE_SYSLOG_H
+					syslog(LOG_DAEMON|LOG_CRIT, "authlib: LDAP_TLS enabled but I'm unable to start tls, check your config\n");
+#endif
 					rc = 1;
-					break;
+				} else {
+#endif
+					switch (ldap_simple_bind_s(bindp, dn, (char *)pass))
+					{
+					case LDAP_SUCCESS:
+						break;
+					case LDAP_INVALID_CREDENTIALS:
+						rc = -1;
+						break;
+					default:
+						rc = 1;
+						break;
+					}
+#if HAVE_LDAP_TLS
 				}
+#endif
 				ldap_unbind(bindp);
 			}
 			if (rc == 0 && newpass)
@@ -944,3 +1167,143 @@ static int auth_ldap_do(const char *user, const char *pass,
 	if (quota)	free(quota);
 	return (rc);
 }
+
+/*
+** Escape a string with special LDAP characters.  Returns NULL if the original
+** string does not have any special LDAP characters (so we don't allocate
+** memory unless absolutely necessary).
+*/
+
+static char *escape_str(const char *user)
+{
+	int i;
+	const char *p;
+	char *q, *r;
+
+	for (i=0, p=user; *p; p++)
+		if (strchr("*()\\", *p))
+			++i;
+
+	if (i == 0)
+		return NULL;	/* No need to escape anything */
+
+	q=malloc(strlen(user)+i+1);
+
+	if (!q)
+	{
+		perror("malloc");
+		exit(1);
+	}
+
+	for (r=q, p=user; *p; p++)
+	{
+		if (strchr("*()\\", *p))
+			*r++= '\\';
+		*r++ = *p;
+	}
+	*r=0;
+	return (q);
+}
+
+/**
+ ** Create an emailmap search string.  I'm going to wrap this into an external
+ ** variable, so I'll use generic coding here.
+ */
+
+struct varlist {
+	const char *varname;
+	int varname_len;
+	const char *varvalue;
+	int varvalue_len;
+} ;
+
+static char *var_expand(const char *, const struct varlist *);
+
+static char *emailmap_get_search_string(const char *str, const char *email)
+{
+	struct varlist vl[3];
+	const char *realmptr=strrchr(email, '@');/* Guaranteed nonNULL */
+
+	static const char userid[]="user";
+	static const char realm[]="realm";
+
+	vl[0].varname=userid;
+	vl[0].varname_len=sizeof(userid)-1;
+	vl[0].varvalue=email;
+	vl[0].varvalue_len=realmptr - email;
+	vl[1].varname=realm;
+	vl[1].varname_len=sizeof(realm)-1;
+	vl[1].varvalue=realmptr+1;
+	vl[1].varvalue_len=strlen(vl[1].varvalue);
+	vl[2].varname=NULL;
+
+	return (var_expand(str, vl));
+}
+
+static char *var_expand(const char *str, const struct varlist *vl)
+{
+	const char *p;
+	int cnt;
+	int pass;
+	char *q, *r;
+
+	cnt=0;
+	q=NULL;
+
+	/*
+	** Pass 1 - count expanded string length, allocate buffer,
+	** Pass 2 - generate the string.
+	*/
+
+	for (pass=0; pass<2; pass++)
+	{
+		if (pass)
+		{
+			if ((q=malloc(cnt)) == NULL)
+				return (NULL);
+		}
+		r=q;
+		cnt=1;
+
+		p=str;
+
+		while (*p)
+		{
+			if (*p == '@')
+			{
+				int j;
+
+				for (j=0; vl[j].varname; j++)
+				{
+					if (memcmp(vl[j].varname, p+1,
+						   vl[j].varname_len) == 0
+					    && p[vl[j].varname_len+1] == '@')
+						break;
+				}
+
+				if (vl[j].varname)
+				{
+					p += vl[j].varname_len+2;
+
+					if (r)
+					{
+						memcpy(r, vl[j].varvalue,
+						       vl[j].varvalue_len);
+						r += vl[j].varvalue_len;
+					}
+					cnt += vl[j].varvalue_len;
+					continue;
+				}
+			}
+
+			if (r)
+				*r++ = *p;
+			++p;
+			++cnt;
+		}
+		if (r)
+			*r=0;
+	}
+
+	return (q);
+}
