@@ -75,7 +75,7 @@
 
 #define	SHARED	"shared"
 
-static const char rcsid[]="$Id: imapd.c,v 1.71 2002/01/08 13:27:18 mrsam Exp $";
+static const char rcsid[]="$Id: imapd.c,v 1.74 2002/04/06 15:48:39 mrsam Exp $";
 
 extern time_t rfc822_parsedt(const char *);
 extern void fetchflags(unsigned long);
@@ -1590,7 +1590,7 @@ int	uid=0;
 
 		writes("*");
 		writes(" QUOTAROOT \"");
-		writeqs(mailbox);
+		writeqs(curtoken->tokenbuf);
 		writes("\" \"");
 		writes(qroot);
 		writes("\"\r\n");
@@ -2043,6 +2043,14 @@ int	uid=0;
 	int	ro=curtoken->tokenbuf[0] == 'E';
 
 		curtoken=nexttoken_nouc();
+
+		if (current_mailbox)
+		{
+			free(current_mailbox);
+			imapscan_free(&current_mailbox_info);
+			current_mailbox=0;
+		}
+
 		mailbox=parse_mailbox_error(tag, curtoken, 0, 1);
 		if ( mailbox == 0)
 			return (0);
@@ -2053,12 +2061,6 @@ int	uid=0;
 			return (-1);
 		}
 
-		if (current_mailbox)
-		{
-			free(current_mailbox);
-			imapscan_free(&current_mailbox_info);
-			current_mailbox=0;
-		}
 		if (imapscan_maildir(&current_mailbox_info, mailbox, 0, ro))
 		{
 			free(mailbox);
@@ -2913,6 +2915,15 @@ const char *tag;
 	fprintf(stderr, "INFO: LOGIN, user=%s, ip=[%s]\n",
 		getenv("AUTHENTICATED"), ip);
 
+	/* We use select() with a timeout, so use non-blocking filedescs */
+
+	if (fcntl(0, F_SETFL, O_NONBLOCK) ||
+	    fcntl(1, F_SETFL, O_NONBLOCK))
+	{
+		perror("fcntl");
+		exit(1);
+	}
+
 	{
 	struct	stat	buf;
 
