@@ -52,7 +52,7 @@
 #include	<openssl/err.h>
 #include	<sys/time.h>
 
-static const char rcsid[]="$Id: starttls.c,v 1.21 2001/09/20 21:44:40 mrsam Exp $";
+static const char rcsid[]="$Id: starttls.c,v 1.25 2002/03/15 19:17:18 mrsam Exp $";
 
 #ifndef NO_RSA
 static RSA *rsa_callback(SSL *, int, int);
@@ -269,8 +269,11 @@ SSL_CTX *ctx;
 
 static int ssl_verify_callback(int goodcert, X509_STORE_CTX *x509)
 {
-	if (peer_verify_domain && !goodcert)
+	if (peer_verify_domain || peer_verify_level)
+	{
+		if (!goodcert)
 			return (0);
+	}
 	return (1);
 }
 
@@ -485,7 +488,11 @@ int	maxfd=sslfd;
 int	suppress_read;
 int	suppress_write;
 
-	if (fcntl(sslfd, F_SETFL, O_NONBLOCK))
+	if (fcntl(sslfd, F_SETFL, O_NONBLOCK)
+	    || fcntl(stdinfd, F_SETFL, O_NONBLOCK)
+	    || fcntl(stdoutfd, F_SETFL, O_NONBLOCK)
+
+	    )
 	{
 		nonsslerror("fcntl");
 		return;
@@ -635,6 +642,12 @@ int	suppress_write;
 
 static int prepsocket(int sockfd)
 {
+	if (fcntl(sockfd, F_SETFL, 0))	/* Turn off O_NONBLOCK, for now */
+	{
+		nonsslerror("fcntl");
+		return (1);
+	}
+
 #ifdef  SO_KEEPALIVE
 	{
 	int	dummy;
