@@ -1,8 +1,8 @@
 /*
-** Copyright 2000 Double Precision, Inc.
+** Copyright 2000-2002 Double Precision, Inc.
 ** See COPYING for distribution information.
 **
-** $Id: big5.c,v 1.5 2001/04/13 01:12:33 mrsam Exp $
+** $Id: big5.c,v 1.7 2002/04/30 20:23:51 mrsam Exp $
 */
 
 #include "big5.h"
@@ -11,9 +11,9 @@
 #include <string.h>
 
 static const unicode_char * const big5fwdlo[]= {
-	big5_a1_lo,
+	NULL,
 	big5_a2_lo,
-	big5_a3_lo,
+	NULL,
 	big5_a4_lo,
 	big5_a5_lo,
 	big5_a6_lo,
@@ -49,7 +49,7 @@ static const unicode_char * const big5fwdlo[]= {
 	big5_c4_lo,
 	big5_c5_lo,
 	big5_c6_lo,
-	big5_c7_lo,
+	NULL,
 	NULL,
 	big5_c9_lo,
 	big5_ca_lo,
@@ -102,9 +102,9 @@ static const unicode_char * const big5fwdlo[]= {
 	big5_f9_lo};
 
 static const unicode_char * const big5fwdhi[]= {
-	big5_a1_hi,
+	NULL,
 	big5_a2_hi,
-	big5_a3_hi,
+	NULL,
 	big5_a4_hi,
 	big5_a5_hi,
 	big5_a6_hi,
@@ -140,7 +140,7 @@ static const unicode_char * const big5fwdhi[]= {
 	big5_c4_hi,
 	big5_c5_hi,
 	big5_c6_hi,
-	big5_c7_hi,
+	NULL,
 	NULL,
 	big5_c9_hi,
 	big5_ca_hi,
@@ -206,8 +206,10 @@ static unicode_char *c2u(const char *cp, int *err)
 
 	for (i=cnt=0; cp[i]; i++)
 	{
-		if ( (int)(unsigned char)cp[i] < 0xA1 ||
+		if ( (int)(unsigned char)cp[i] <= 0xA1 ||
+		     (int)(unsigned char)cp[i] == 0xA3 ||
 		     (int)(unsigned char)cp[i] > 0xF9 ||
+		     (int)(unsigned char)cp[i] == 0xC7 ||
 		     (int)(unsigned char)cp[i] == 0xC8 ||
 		     cp[i+1] == 0)
 		{
@@ -228,28 +230,28 @@ static unicode_char *c2u(const char *cp, int *err)
 	{
 		int a=(int)(unsigned char)cp[i], b;
 
-		if ( a >= 0xA1 && a <= 0xF9 && a != 0xC8 && cp[i+1])
+		if ( a > 0xA1 && a <= 0xF9 && a != 0xA3
+		     && a != 0xC7 && a != 0xC8 && cp[i+1])
 		{
 			b=(int)(unsigned char)cp[i+1];
 
-			if (b >= 64 && b < 128)
+			++i;
+			if (b >= 64 && b < 128 && big5fwdlo[a-0xa1])
 			{
 				unicode_char ucv=big5fwdlo[a-0xa1][b-64];
 
 				if (ucv)
 				{
-					++i;
 					uc[cnt++]= ucv;
 					continue;
 				}
 			}
-			else if (b >= 161 && b < 256)
+			else if (b >= 161 && b < 256 && big5fwdhi[a-0xa1])
 			{
 				unicode_char ucv=big5fwdhi[a-0xa1][b-161];
 
 				if (ucv)
 				{
-					++i;
 					uc[cnt++]= ucv;
 					continue;
 				}
@@ -435,7 +437,7 @@ const struct unicode_info unicode_BIG5 = {
 
 int main()
 {
-	FILE *fp=fopen("big5.txt", "r");
+	FILE *fp=popen("gunzip -cd <Unihan-3.2.0.txt.gz", "r");
 	char buf[4000];
 	unicode_char *uc;
 	char *s, *p;
@@ -449,7 +451,7 @@ int main()
 		unsigned a, b, c;
 		int dummy;
 
-		if (sscanf(buf, "%x %x", &a, &b) != 2)
+		if (sscanf(buf, "U+%4x kBigFive %4x", &b, &a) != 2)
 			continue;
 		printf("0x%04x 0x%04x: ", a, b);
 
@@ -541,13 +543,13 @@ int main()
 	free(uc);
 	if (!s)
 	{
-		printf("us-asccu u2c failure\n");
+		printf("us-ascii u2c failure\n");
 		return (1);
 	}
 	free(s);
 
-	buf[0]=0x40;
-	buf[1]=0xF0;
+	buf[0]=0xA2;
+	buf[1]=0x40;
 	buf[2]=0;
 
 	uc=c2u(buf, NULL);
@@ -571,8 +573,8 @@ int main()
 	       (int)(unsigned char)s[1]);
 	free(s);
 
-	buf[0]=0x40;
-	buf[1]=0xF0;
+	buf[0]=0xA2;
+	buf[1]=0x40;
 	buf[2]=0;
 
 	uc=c2u(buf, &dummyi);
