@@ -2,7 +2,7 @@
 ** Copyright 2000 Double Precision, Inc.
 ** See COPYING for distribution information.
 **
-** $Id: gb2312.c,v 1.4 2001/04/13 21:07:38 mrsam Exp $
+** $Id: gb2312.c,v 1.6 2002/04/30 20:23:51 mrsam Exp $
 */
 
 #include "gb2312.h"
@@ -11,21 +11,6 @@
 #include <string.h>
 
 static const unicode_char * const gb2312[]= {
-	gb2312_a1,
-	gb2312_a2,
-	gb2312_a3,
-	gb2312_a4,
-	gb2312_a5,
-	gb2312_a6,
-	gb2312_a7,
-	gb2312_a8,
-	gb2312_a9,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
 	gb2312_b0,
 	gb2312_b1,
 	gb2312_b2,
@@ -113,10 +98,8 @@ static unicode_char *c2u(const char *cp, int *err)
 
 	for (i=cnt=0; cp[i]; i++)
 	{
-		if ( (int)(unsigned char)cp[i] < 0xA1 ||
+		if ( (int)(unsigned char)cp[i] < 0xB0 ||
 		     (int)(unsigned char)cp[i] > 0xF7 ||
-		     ((int)(unsigned char)cp[i] > 0xA9 &&
-		      (int)(unsigned char)cp[i] < 0xB0) ||
 		     cp[i+1] == 0)
 		{
 			++cnt;
@@ -137,18 +120,17 @@ static unicode_char *c2u(const char *cp, int *err)
 	{
 		int a=(int)(unsigned char)cp[i], b;
 
-		if ( a >= 0xA1 && a <= 0xF7 && (a <= 0xA9 || a >= 0xB0) &&
-		     cp[i+1])
+		if ( a >= 0xB0 && a <= 0xF7 && cp[i+1])
 		{
 			b=(int)(unsigned char)cp[i+1];
+			++i;
 
 			if (b >= 0xa1 && b < 0xFF)
 			{
-				unicode_char ucv=gb2312[a-0xa1][b-0xa1];
+				unicode_char ucv=gb2312[a-0xb0][b-0xa1];
 
 				if (ucv)
 				{
-					++i;
 					uc[cnt++]= ucv;
 					continue;
 				}
@@ -334,7 +316,7 @@ const struct unicode_info unicode_GB2312 = {
 
 int main()
 {
-	FILE *fp=fopen("gb2312.txt", "r");
+	FILE *fp=popen("gunzip -cd <Unihan-3.2.0.txt.gz", "r");
 	char buf[4000];
 	unicode_char *uc;
 	char *s, *p;
@@ -348,7 +330,7 @@ int main()
 		unsigned a, b, c;
 		int dummy;
 
-		if (sscanf(buf, "%x %x", &a, &b) != 2)
+		if (sscanf(buf, "U+%4x kIRG_GSource 0-%4x", &b, &a) != 2)
 			continue;
 		a |= 0x8080;
 
@@ -466,7 +448,7 @@ int main()
 	       (int)(unsigned char)s[1]);
 	free(s);
 
-	buf[0]=0xA1;
+	buf[0]=0xB2;
 	buf[1]=0x20;
 	buf[2]=0;
 
